@@ -38,6 +38,8 @@
 #include "packages.h"
 #include "serial.h"
 #include "cardinfo.h"
+#include "keyboard.h"
+#include "timeanddate.h"
 
 static GtkWidget *passwd_entry;
 static int retv;
@@ -82,25 +84,11 @@ update_dhcp_status (const gchar * active)
 
 	if (use_dhcp)
 	{
-		change_cfg_value ("/etc/pcmcia/network.opts", "DHCP", "\"y\"",
-				  '=');
-		switch (fork ())
-		{
-		case -1:
-			fprintf (stderr, "cant fork\n");
-			exit (errno);
-		case 0:
-			execlp ("/sbin/dhcpcd", "dhcpcd");
-			exit (0);
-		default:
-			break;
-		}
+		change_cfg_value ("/etc/pcmcia/network.opts", "DHCP", "\"y\"",'=');
 	}
 	else
 	{
-		change_cfg_value ("/etc/pcmcia/network.opts", "DHCP", "\"n\"",
-				  '=');
-		system ("/usr/bin/killall dhcpcd");
+		change_cfg_value ("/etc/pcmcia/network.opts", "DHCP", "\"n\"",'=');
 	}
 }
 
@@ -113,7 +101,8 @@ update_dns_server (const gchar * server)
 void
 update_time_from_net (const gchar * server)
 {
-	if (system_printf ("/usr/sbin/ntpdate -b %s", server))
+	char *tstr = g_strdup_printf ("/usr/sbin/ntpdate -b %s", server);
+	if (system(tstr))
 	{
 		fprintf (stderr, "failed to execute ntpdate\n");
 	}
@@ -121,9 +110,14 @@ update_time_from_net (const gchar * server)
 	{
 		system ("echo > /var/spool/at/trigger");
 	}
+	g_free(tstr);
 }
 
 
+/* Changes a setting in a config file, to be used with caution.
+ * seperator == 0 means to comment out this line 
+ */
+
 void
 change_cfg_value (const gchar * file, const gchar * var, const gchar * val,
 		  gchar seperator)
@@ -160,21 +154,29 @@ change_cfg_value (const gchar * file, const gchar * var, const gchar * val,
 		    && (!g_str_has_prefix (g_strchomp (lines[i]), "#")))
 		{
 			delim = lines[i];
-			j = get_first_char (delim);
-			if (j > 0)
+			if (seperator)
 			{
-				tmpval = g_malloc (j);
-				strncpy (tmpval, delim, j);
-				lines[i] =
-					g_strdup_printf ("%s%s%c%s", tmpval,
-							 var, seperator, val);
-				g_free (tmpval);
+				j = get_first_char (delim);
+				if (j > 0)
+				{
+					tmpval = g_malloc (j+2);
+					strncpy (tmpval, delim, j);
+					tmpval[j]=0;
+					lines[i] =
+						g_strdup_printf ("%s%s%c%s", tmpval,
+								 var, seperator, val);
+					g_free (tmpval);
+				}
+				else
+				{
+					lines[i] =
+						g_strdup_printf ("%s%c%s", var,
+								 seperator, val);
+				}
 			}
 			else
 			{
-				lines[i] =
-					g_strdup_printf ("%s%c%s", var,
-							 seperator, val);
+				lines[i] = g_strdup_printf ("# %s", lines[i]);
 			}
 		}
 		i++;
@@ -246,58 +248,6 @@ update_login_background (char *file)
 		g_warning ("Could not create link to %s\n", file);
 }
 
-void
-set_timezone (gchar * zone)
-{
-	gchar *profile;
-	gchar **proflines;
-	gint length;
-	gchar *delim;
-	FILE *profnew;
-	gint i = 0;
-	gint j = 0;
-
-	GError *err = NULL;
-
-	delim = g_strdup ("\n");
-	g_file_get_contents ("/etc/profile", &profile, &length, &err);
-	proflines = g_strsplit (profile, delim, 255);
-	g_free (delim);
-	delim = NULL;
-	g_free (profile);
-
-	while (proflines[i])
-	{
-		if ((g_strrstr (proflines[i], "TZ"))
-		    && (g_strrstr (proflines[i], "export"))
-		    && (!g_strrstr (proflines[i], "#")))
-		{
-			delim = proflines[i];
-			proflines[i] = g_strdup_printf ("export TZ=%s", zone);
-		}
-		i++;
-	}
-
-	i--;
-
-	if (delim == NULL)
-	{
-		proflines = realloc (proflines, i * sizeof (gchar *));
-		proflines[i] = g_strdup_printf ("export TZ=%s", zone);
-		i++;
-	}
-	else
-		free (delim);
-
-	profnew = fopen ("/etc/profile", "w");
-
-	for (j = 0; j < i; j++)
-	{
-		fprintf (profnew, "%s\n", proflines[j]);
-	}
-	fclose (profnew);
-	g_strfreev (proflines);
-}
 
 /* 
 	Creating a new homedir for a user.
@@ -337,7 +287,7 @@ check_user_access (const char *cmd)
 		know_global_user_access = TRUE;
 	}
 
-	// allow screen settings
+	/* allow screen settings */
 	if (!strcmp (cmd, "SCRR"))
 		return TRUE;
 	if (!strcmp (cmd, "SCRB"))
@@ -345,6 +295,11 @@ check_user_access (const char *cmd)
 	if (!strcmp (cmd, "SCRL"))
 		return TRUE;
 
+	/* allow time settings */
+	if (!strcmp (cmd, "STIM"))
+		return TRUE;
+	if (!strcmp (cmd, "NTPD"))
+		return TRUE;
 	return global_user_access;
 }
 
@@ -358,9 +313,10 @@ suidloop (int write, int read)
 	char passwd[100];
 	FILE *in = fdopen (read, "r");
 	char *bin = NULL;
-	char arg1[100];
+	char arg1[255];
 	char arg2[100];
 	int numarg = 0;
+	int numarg2 = 0;
 
 	setuid (0);
 	nsreturnfd = write;
@@ -379,14 +335,13 @@ suidloop (int write, int read)
 			bin = NULL;
 			if ((check_user_access (cmd) == TRUE) || (check_root_passwd (passwd)))	// we want to know it exact
 			{
-				/* if (strcmp (cmd, "CHEK") == 0)  we do nothing - just check root password */
 
-				if (strcmp (cmd, "NTPD") == 0)
+				if (strcmp (cmd, "NTPD") == 0)  // sets system time from timeserver
 				{
 					fscanf (in, "%100s", arg2);	// get timeserver
 					update_time_from_net (arg2);
 				}
-				else if (strcmp (cmd, "STIM") == 0)
+				else if (strcmp (cmd, "STIM") == 0)  // sets the system time
 				{
 					time_t t;
 					fscanf (in, "%ld", &t);
@@ -401,7 +356,7 @@ suidloop (int write, int read)
 				}
 				/* of course it is a security hole */
 				/* but certainly enough for PDA..  */
-				else if (strcmp (cmd, "CPPW") == 0)
+				else if (strcmp (cmd, "CPPW") == 0)  // installs a new passwd file
 				{
 					bin = "/bin/cp";
 					strcpy (arg1, "/tmp/passwd");
@@ -412,7 +367,7 @@ suidloop (int write, int read)
 					system_printf ("chmod 0644 %s", arg2);
 					system_printf ("/bin/rm -f %s", arg1);
 				}
-				else if (strcmp (cmd, "CPIF") == 0)
+				else if (strcmp (cmd, "CPIF") == 0)  // installs a new interfaces file
 				{
 					fscanf (in, "%100s", arg2);	// to forget soon...
 					bin = "/bin/cp";
@@ -427,7 +382,7 @@ suidloop (int write, int read)
 					system_printf ("/bin/rm -f %s", arg1);
 					system ("/etc/init.d/networking restart");
 				}
-				else if (strcmp (cmd, "CPOI") == 0)
+				else if (strcmp (cmd, "CPOI") == 0)  // rewrites owner information data
 				{
 					bin = "/bin/cp";
 					system_printf ("/bin/cp %s %s",
@@ -438,131 +393,145 @@ suidloop (int write, int read)
 					system_printf ("/bin/rm -f %s",
 						       GPE_OWNERINFO_TMP);
 				}
-				else if (strcmp (cmd, "XCAL") == 0)
+				else if (strcmp (cmd, "XCAL") == 0)  // runs xcalibrate
 				{
-					system ("/usr/X11R6/bin/xcalibrate");
+					system ("/usr/bin/gpe-xcalibrate.sh");
 				}
-				else if (strcmp (cmd, "STZO") == 0)
+				else if (strcmp (cmd, "STZO") == 0)  // changes the timezone setting 
 				{
 					fscanf (in, "%100s", arg2);
-					set_timezone (arg2);
+					set_timezone ("/etc/profile",arg2);
 				}
-				else if (strcmp (cmd, "ULBS") == 0)
+				else if (strcmp (cmd, "ULBS") == 0)  // turn login bg on/off
 				{
 					fscanf (in, "%1s", arg2);
 					update_login_bg_sh (arg2);
 				}
-				else if (strcmp (cmd, "UOIS") == 0)
+				else if (strcmp (cmd, "UOIS") == 0)  // change if owner information is shown or not
 				{
 					fscanf (in, "%1s", arg2);
 					update_ownerinfo_sh (arg2);
 				}
-				else if (strcmp (cmd, "ULDS") == 0)
+				else if (strcmp (cmd, "ULDS") == 0)  // change auto lock of display
 				{
 					fscanf (in, "%1s", arg2);
 					update_login_lock_auto (arg2);
 				}
-				else if (strcmp (cmd, "ULBF") == 0)
+				else if (strcmp (cmd, "ULBF") == 0)  // change background of login screen
 				{
 					fscanf (in, "%100s", arg2);
 					update_login_background (arg2);
 				}
-				else if (strcmp (cmd, "DHCP") == 0)
+				else if (strcmp (cmd, "DHCP") == 0)  // switch dhcp usage on/off
 				{
 					fscanf (in, "%100s", arg2);
 					update_dhcp_status (arg2);
 				}
-				else if (strcmp (cmd, "SDNS") == 0)
+				else if (strcmp (cmd, "SDNS") == 0)  // change to another dns server
 				{
 					fscanf (in, "%100s", arg2);
 					update_dns_server (arg2);
 				}
-				else if (strcmp (cmd, "SCRB") == 0)
+				else if (strcmp (cmd, "SCRB") == 0)  // change frontlight brightness
 				{
 					fscanf (in, "%d", &numarg);
 					update_screen_brightness (numarg);
 				}
-				else if (strcmp (cmd, "SCRR") == 0)
+				else if (strcmp (cmd, "SCRR") == 0)  // rotate screen
 				{
 					fscanf (in, "%d", &numarg);
 					update_screen_rotation (numarg);
 				}
-				else if (strcmp (cmd, "SCRL") == 0)
+				else if (strcmp (cmd, "SCRL") == 0)  // chnage frontlight state
 				{
 					fscanf (in, "%d", &numarg);
 					update_light_status (numarg);
 				}
-				else if (strcmp (cmd, "CRHD") == 0)
+				else if (strcmp (cmd, "CRHD") == 0)  // create a users home directory
 				{
 					fscanf (in, "%100s", arg1);	// directory
 					fscanf (in, "%100s", arg2);	// username
 					create_homedir (arg1, arg2);
 				}
-				else if (strcmp (cmd, "NWUD") == 0)
+				else if (strcmp (cmd, "NWUD") == 0)  // run system packages update
 				{
 					fscanf (in, "%100s", arg1);
 					update_packages ();
 				}
-				else if (strcmp (cmd, "SERU") == 0)
+				else if (strcmp (cmd, "NWIS") == 0)  // run package installation
+				{
+					fscanf (in, "%100s", arg1);
+					do_package_install (arg1,FALSE);
+				}
+				else if (strcmp (cmd, "NWRM") == 0)  // run package uninstall
+				{
+					fscanf (in, "%100s", arg1);
+					do_package_install (arg1,TRUE);
+				}
+				else if (strcmp (cmd, "PAIS") == 0)  // install a ipk package
+				{
+					fscanf (in, "%100s", arg1);
+					do_package_install (arg1,FALSE);
+				}
+				else if (strcmp (cmd, "SERU") == 0)  // change serial port usage
 				{
 					fscanf (in, "%d", &numarg);
 					assign_serial_port (numarg);
 				}
-				else if (strcmp (cmd, "CMID") == 0)
+				else if (strcmp (cmd, "SGPS") == 0)  // change gps settings
+				{
+					fscanf (in, "%100s", arg1);
+					update_gpsd_settings(arg1+1, (arg1[0] == '1') ? 1 : 0);
+				}
+				else if (strcmp (cmd, "CMID") == 0)  // identify pcmcia card
 				{
 					fscanf (in, "%d", &numarg);
 					do_get_card_ident (numarg);
 				}
-				else if (strcmp (cmd, "CMRE") == 0)
+				else if (strcmp (cmd, "CMCO") == 0)  // do pcmcia ioctl
 				{
-					fscanf (in, "%s", arg1);
+					fscanf (in, "%d %d", &numarg, &numarg2);
+					do_ioctl (numarg,numarg2);
+				}
+				else if (strcmp (cmd, "CMRE") == 0)  // reset cardmgr
+				{
+					fscanf (in, "%100s", arg1);
 					do_reset ();
 				}
-				else if (strcmp (cmd, "CMCP") == 0)
+				else if (strcmp (cmd, "CMCP") == 0)  // copy pcmcia config
 				{
-					fscanf (in, "%s", arg1);
-					system_printf ("/bin/cp %s %s",
+					fscanf (in, "%d", &numarg); // numarg==1 tells us the destination is wlan-ng.conf
+					if (numarg)
+					{
+						system_printf ("/bin/cp %s %s",
+						       pcmcia_tmpcfgfile,
+						       wlan_ng_cfgfile);
+						system_printf ("chmod 0644 %s",
+						       wlan_ng_cfgfile);
+						system_printf ("/bin/rm -f %s",
+						       pcmcia_tmpcfgfile);
+					}
+					else
+					{
+						system_printf ("/bin/cp %s %s",
 						       pcmcia_tmpcfgfile,
 						       pcmcia_cfgfile);
-					system_printf ("chmod 0644 %s",
+						system_printf ("chmod 0644 %s",
 						       pcmcia_cfgfile);
-					system_printf ("/bin/rm -f %s",
+						system_printf ("/bin/rm -f %s",
 						       pcmcia_tmpcfgfile);
-				}
-#if 0
-				if (bin)	// fork and exec
-				{
-					int PID;
-					switch (PID = fork ())
-					{
-					case -1:
-						fprintf (stderr,
-							 "cant fork\n");
-						exit (errno);
-					case 0:
-						switch (numarg)
-						{
-						case 1:
-							execlp (bin,
-								strrchr (bin,
-									 '/')
-								+ 1, arg1,
-								NULL);
-							break;
-						case 2:
-							execlp (bin,
-								strrchr (bin,
-									 '/')
-								+ 1, arg1,
-								arg2, NULL);
-							break;
-						}
-						exit (0);
-					default:
-						break;
 					}
 				}
-#endif
+				else if (strcmp (cmd, "KBDC") == 0)  // write a new keyboard config file
+				{
+					fscanf (in, "%255s", arg1);
+					write_keyboard_cfg (arg1);
+				}
+				else if (strcmp (cmd, "GAUA") == 0)  // change user access to gpe-conf
+				{
+					fscanf (in, "%100s", arg1);
+					do_change_user_access (arg1);
+				}
 			}	// if check_user_access
 			else	// clear buffer
 			{
@@ -570,7 +539,7 @@ suidloop (int write, int read)
 				{
 					fscanf (in, "%100s", arg2);
 				}
-				else if (strcmp (cmd, "STIM") == 0)
+				else if (strcmp (cmd, "STIM") == 0) 
 				{
 					time_t t;
 					fscanf (in, "%ld", &t);
