@@ -1,3 +1,19 @@
+/*
+ * gpe-conf
+ *
+ * Copyright (C) 2002  Pierre TARDY <tardyp@free.fr>
+ *	             2003  Florian Boor <florian.boor@kernelconcepts.de>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version
+ * 2 of the License, or (at your option) any later version.
+ *
+ * Time and date settings module.
+ *
+ */
+
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,6 +41,7 @@
 #include <gpe/spacing.h>
 #include <gpe/errorbox.h>
 #include <gpe/gtkdatecombo.h>
+#include <gpe/gpetimesel.h>
 
 
 /* --- local types and constants --- */
@@ -33,14 +50,14 @@ gchar *Ntpservers[6]=
   {
     "time.handhelds.org",
     "time.apple.com",
-    "time.esec.com.au",
+    "ntp2.uni-siegen.de",
     "ptbtime1.ptb.de",
     "ntp2c.mcc.ac.uk",
     "ntppub.tamu.edu"
   };
 
 gchar *timezones[][3]= {
-    {"UTC / WET / GMT","UTC","WEDST"},
+    {"UTC / WET / GMT","UTC-0","WEDST"},
     {"MET / MEZ / CET / SWT","MEZ-1","MESZ"},
     {"EET","EET-2","EETD"},
     {"BT","BT-3","BT"},
@@ -58,10 +75,9 @@ gchar *timezones[][3]= {
     {"EST","EST+5","EDT"},
     {"AST","AST+4","ADT"},
     {"WAT","WAT+1","WDT"},
-	{"<advanced>","<>",""}
   };
 
-#define TZ_MAXINDEX 19
+#define TZ_MAXINDEX 18
 
 typedef struct 
 {
@@ -105,14 +121,11 @@ static struct
   GtkWidget *controlvbox4;
   GtkWidget *cal;
   GtkWidget *hbox;
-  GtkWidget *h;
-  GtkWidget *m;
-  GtkWidget *s;
+  GtkWidget *tsel;
   GtkWidget *ntpserver;
   GtkWidget *internet;
   GtkWidget *timezone;
   GtkWidget *usedst;
-  GtkWidget *advanced;
   GtkWidget *dsth;
   GtkWidget *dstm;
   GtkWidget *defaultdst;
@@ -121,7 +134,11 @@ static struct
   GtkWidget *offsetlabel;
 } self;
 
-int trc = 0;  // countdown for waiting for time change
+static int trc = 0;  // countdown for waiting for time change
+static int tid = 0;  // timeout id
+static int isdst;    // is a dst active?
+static int need_warning = FALSE;
+static int nonroot_mode = FALSE;
 
 /* --- local intelligence --- */
 
@@ -136,8 +153,6 @@ static void update_enabled_widgets(GtkWidget *sender)
 	gtk_widget_set_sensitive(self.dstm,(usedst && !defaultdst));
 	gtk_widget_set_sensitive(self.dstlm,(usedst && !defaultdst));
 	gtk_widget_set_sensitive(self.dstlh,(usedst && !defaultdst));
-
-	gtk_widget_set_sensitive(self.advanced,FALSE);		
 }
 
 /*
@@ -173,15 +188,69 @@ char *compose_tz_advanced(tzinfo tzi)
 }
 
 
-/*
- *  This function composes a TZ string from given parameters.
- */
-char *compose_tz_simple(char *tz,char *dst)
+/* This finction sets TZ in given file to a new value */
+
+void
+set_timezone (gchar *fprofile, gchar *zone)
 {
-	char *result;
+	gchar *profile;
+	gchar **proflines = NULL;
+	gint length;
+	gchar *delim;
+	FILE *profnew;
+	gint i = 0;
+	gint j = 0;
+
+	GError *err = NULL;
+
+	if (!access(fprofile,F_OK))
+	{
+		delim = g_strdup ("\n");
+		g_file_get_contents (fprofile, &profile, &length, &err);
+		proflines = g_strsplit (profile, delim, 255);
+		g_free (delim);
+		g_free (profile);
+	}
+	else
+	{
+		proflines = g_malloc(2 * sizeof(char*));
+		proflines[0] = g_strdup("# File generated by gpe-conf, please edit :-)");
+		proflines[1] = NULL;
+	}
+	delim = NULL;
 	
-	result = g_strdup_printf("%s%s",tz,dst);
-	return(result);
+	while (proflines[i])
+	{
+		if ((g_strrstr (proflines[i], "TZ"))
+		    && (g_strrstr (proflines[i], "export"))
+		    && (!g_strrstr (proflines[i], "#")))
+		{
+			delim = proflines[i];
+			proflines[i] = g_strdup_printf ("export TZ=%s", zone);
+		}
+		i++;
+	}
+
+	if (delim == NULL)
+	{
+		proflines = realloc (proflines, (i+2) * sizeof (gchar *));
+		proflines[i] = g_strdup_printf ("export TZ=%s", zone);
+		proflines[i+1] = NULL;
+		i++;
+	}
+	else
+		free (delim);
+
+	profnew = fopen (fprofile, "w");
+
+	for (j = 0; j < i; j++)
+	{
+		fprintf (profnew, "%s\n", proflines[j]);
+	}
+	fchmod(fileno(profnew),S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+	fclose (profnew);
+	
+	g_strfreev (proflines);
 }
 
 
@@ -198,7 +267,7 @@ tzinfo get_tz_info(char *tzstr)
 	
 	/* init with defaults */
 	sprintf(result.tzname,"UTC");
-	sprintf(result.dstname,"");
+	result.dstname[0] = 0;
 	result.utcofs_h = 0;
 	result.utcofs_m = 0;
 	result.utcdstofs_h = 0;
@@ -290,12 +359,16 @@ tzinfo get_tz_info(char *tzstr)
 	return(result);
 }
 
-gint refresh_time()
+gboolean refresh_time()
 {
 	Time_Restore();
 	trc--;
-	if (!trc)  gtk_widget_set_sensitive(self.internet,TRUE);
-	return trc;
+	if (!trc)  
+	{
+		gtk_widget_set_sensitive(self.internet,TRUE);
+		gtk_timeout_remove(tid);
+	}
+	return (trc ? TRUE : FALSE);
 }
 
 void GetInternetTime()
@@ -303,18 +376,28 @@ void GetInternetTime()
   gtk_widget_set_sensitive(self.internet,FALSE);
   suid_exec("NTPD",gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (self.ntpserver)->entry)));
   trc = 10;
-  gtk_timeout_add(500,refresh_time,NULL);
+  tid = gtk_timeout_add(500,refresh_time,NULL);
+}
+
+
+static void
+do_tabchange (GtkNotebook * notebook,
+	      GtkNotebookPage * page, guint page_num, gpointer user_data)
+{
+	if (page_num == 1)
+		need_warning = TRUE;
 }
 
 
 /* --- gpe-conf interface --- */
 
-GtkWidget *Time_Build_Objects()
+GtkWidget *Time_Build_Objects(gboolean nonroot)
 {  
   guint idx;
   GList *ntpsrv = NULL;
   GList *tzones = NULL;
  
+  GtkWidget *table;
   GtkObject *adj;
   GtkTooltips *tooltips;
   GtkWidget *notebook, *label, *hbox;
@@ -323,8 +406,6 @@ GtkWidget *Time_Build_Objects()
   struct tm ts;     /* gtk_cal seems to modify the ptr
 				returned by localtime, so we duplicate it.. */
 
-  guint gpe_catspacing = gpe_get_catspacing ();
-  gchar *gpe_catindent = gpe_get_catindent ();
   guint gpe_border     = gpe_get_border ();
   guint gpe_boxspacing = gpe_get_boxspacing ();
   
@@ -333,6 +414,8 @@ GtkWidget *Time_Build_Objects()
   
   tzinfo tzi;
   
+  nonroot_mode = nonroot;
+  
   // get the time and the date.
   time(&t);
   tsptr = localtime(&t);
@@ -341,152 +424,112 @@ GtkWidget *Time_Build_Objects()
 
   if(ts.tm_year < 2002)
     ts.tm_year=2002;
+  isdst = ts.tm_isdst;
 
   tooltips = gtk_tooltips_new ();
   
   notebook = gtk_notebook_new();
   gtk_container_set_border_width (GTK_CONTAINER (notebook), gpe_border);
 
-  label = gtk_label_new(_("Time & Date"));
-  self.categories = gtk_vbox_new (FALSE, gpe_catspacing);
-  
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),self.categories,label);
-  
-  gtk_object_set_data(GTK_OBJECT(notebook),"tooltips",tooltips);
-  
-  gtk_container_set_border_width (GTK_CONTAINER (self.categories), 0);
-
-  /* -------------------------------------------------------------------------- */
-  self.catvbox1 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.categories), self.catvbox1, TRUE, TRUE, 0);
-
-  self.catlabel1 = gtk_label_new (NULL); 
-  fstr = g_strdup_printf("%s %s %s","<b>",_("Date"),"</b>");
-  gtk_label_set_markup (GTK_LABEL(self.catlabel1),fstr); 
-  g_free(fstr);
-  gtk_box_pack_start (GTK_BOX (self.catvbox1), self.catlabel1, FALSE, FALSE, 0);
-  gtk_label_set_justify (GTK_LABEL (self.catlabel1), GTK_JUSTIFY_LEFT);
-  gtk_misc_set_alignment (GTK_MISC (self.catlabel1), 0, 0.5);
-
-  self.catconthbox1 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (self.catvbox1), self.catconthbox1, TRUE, TRUE, 0);
-
-  self.catindentlabel1 = gtk_label_new (gpe_catindent);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox1), self.catindentlabel1, FALSE, FALSE, 0);
-
-  self.controlvbox1 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox1), self.controlvbox1, TRUE, TRUE, 0);
-  
-  self.cal = gtk_date_combo_new ();
-  gtk_calendar_select_month (GTK_CALENDAR (GTK_DATE_COMBO(self.cal)->cal), ts.tm_mon, ts.tm_year);
-  gtk_calendar_select_day (GTK_CALENDAR (GTK_DATE_COMBO(self.cal)->cal), ts.tm_mday);
-  
-  gtk_box_pack_start (GTK_BOX (self.controlvbox1), self.cal, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip (tooltips, self.cal, _("Enter current date here or use button to select."), NULL);
-
-  /* -------------------------------------------------------------------------- */
-  self.catvbox2 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.categories), self.catvbox2, TRUE, TRUE, 0);
-
-  self.catlabel2 = gtk_label_new (NULL); 
-  fstr = g_strdup_printf("%s %s %s","<b>",_("Time"),"</b>");
-  gtk_label_set_markup (GTK_LABEL(self.catlabel2),fstr); 
-  g_free(fstr);
-  gtk_box_pack_start (GTK_BOX (self.catvbox2), self.catlabel2, FALSE, FALSE, 0);
-  gtk_label_set_justify (GTK_LABEL (self.catlabel2), GTK_JUSTIFY_LEFT);
-  gtk_misc_set_alignment (GTK_MISC (self.catlabel2), 0, 0.5);
-
-  self.catconthbox2 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (self.catvbox2), self.catconthbox2, TRUE, TRUE, 0);
-
-  self.catindentlabel2 = gtk_label_new (gpe_catindent);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox2), self.catindentlabel2, FALSE, FALSE, 0);
-
-  self.controlvbox2 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox2), self.controlvbox2, TRUE, TRUE, 0);
-  
-  self.hbox = gtk_hbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.controlvbox2), self.hbox, FALSE, FALSE, 0);
-
-  adj = gtk_adjustment_new(ts.tm_hour,0,24,1,6,6);
-  self.h = gtk_spin_button_new (GTK_ADJUSTMENT(adj),1,0);
-  gtk_container_add(GTK_CONTAINER(self.hbox),self.h);
-
-  adj = gtk_adjustment_new(ts.tm_min,0,60,1,15,15);
-  self.m = gtk_spin_button_new (GTK_ADJUSTMENT(adj),1,0);
-  gtk_container_add(GTK_CONTAINER(self.hbox),self.m);
-
-  adj = gtk_adjustment_new(ts.tm_sec,0,60,1,15,15);
-  self.s = gtk_spin_button_new (GTK_ADJUSTMENT(adj),1,0);
-  gtk_container_add(GTK_CONTAINER(self.hbox),self.s);
-
-  /* -------------------------------------------------------------------------- */
-
-  self.catvbox3 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.categories), self.catvbox3, TRUE, TRUE, 0);
-
-  self.catlabel3 = gtk_label_new (NULL);
-  fstr = g_strdup_printf("%s %s %s","<b>",_("Network"),"</b>");
-  gtk_label_set_markup (GTK_LABEL(self.catlabel3),fstr); 
-  g_free(fstr);
-  
-  gtk_box_pack_start (GTK_BOX (self.catvbox3), self.catlabel3, FALSE, FALSE, 0);
-  gtk_label_set_justify (GTK_LABEL (self.catlabel3), GTK_JUSTIFY_LEFT);
-  gtk_misc_set_alignment (GTK_MISC (self.catlabel3), 0, 0.5);
-
-  self.catconthbox3 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (self.catvbox3), self.catconthbox3, TRUE, TRUE, 0);
-
-  self.catindentlabel3 = gtk_label_new (gpe_catindent);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox3), self.catindentlabel3, FALSE, FALSE, 0);
-
-  self.controlvbox3 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox3), self.controlvbox3, TRUE, TRUE, 0);
-
-
-  self.ntpserver = gtk_combo_new ();
-
-  for (idx=0; idx<5; idx++) {
-    ntpsrv = g_list_append (ntpsrv, Ntpservers[idx]);
-  }
-
-  gtk_combo_set_popdown_strings (GTK_COMBO (self.ntpserver), ntpsrv);
-  gtk_box_pack_start(GTK_BOX(self.controlvbox3), self.ntpserver, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip (tooltips, self.ntpserver, _("Here you may select which timeserver i use to set my clock."), NULL);
-
-  self.internet = gtk_button_new_with_label(_("Get time from network"));
-  gtk_signal_connect (GTK_OBJECT(self.internet), "clicked",
-		      (GtkSignalFunc) GetInternetTime, NULL);
-  gtk_widget_set_sensitive(self.internet,is_network_up());
-  
-  gtk_box_pack_start(GTK_BOX(self.controlvbox3), self.internet, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip (tooltips, self.internet, _("If i'm connected to the Internet, you may press this butten to set the time on this device using the timeserver above."), NULL);
-
+  if (!nonroot_mode)
+  {
+	  label = gtk_label_new(_("Time & Date"));
+	  self.categories =  table = gtk_table_new (6, 3, FALSE);
+	  gtk_container_set_border_width (GTK_CONTAINER (table), gpe_border);
+	  gtk_table_set_row_spacings (GTK_TABLE (table), gpe_boxspacing);
+	  gtk_table_set_col_spacings (GTK_TABLE (table), gpe_boxspacing);
+	  gtk_widget_set_size_request(notebook,140,-1);
+	  
+	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),self.categories,label);
+	  
+	  gtk_object_set_data(GTK_OBJECT(notebook),"tooltips",tooltips);
+	  
+	  gtk_container_set_border_width (GTK_CONTAINER (self.categories), 0);
+	
+	  /* -------------------------------------------------------------------------- */
+	
+	  self.catlabel1 = gtk_label_new (NULL); 
+	  fstr = g_strdup_printf("%s %s %s","<b>",_("Date"),"</b>");
+	  gtk_label_set_markup (GTK_LABEL(self.catlabel1),fstr); 
+	  g_free(fstr);
+	  gtk_table_attach (GTK_TABLE (table), self.catlabel1, 0, 3, 0, 1,
+				GTK_FILL  | GTK_EXPAND,GTK_FILL  | GTK_EXPAND,0,0);
+	
+	  gtk_misc_set_alignment (GTK_MISC (self.catlabel1), 0.0, 0.9);
+	  
+	  self.cal = gtk_date_combo_new ();
+	  gtk_calendar_select_month (GTK_CALENDAR (GTK_DATE_COMBO(self.cal)->cal), ts.tm_mon, ts.tm_year);
+	  gtk_calendar_select_day (GTK_CALENDAR (GTK_DATE_COMBO(self.cal)->cal), ts.tm_mday);
+	  gtk_table_attach (GTK_TABLE (table), self.cal, 0, 3, 1, 2,
+				GTK_FILL,0,3,0);
+	  
+	  gtk_tooltips_set_tip (tooltips, self.cal, _("Enter current date here or use button to select."), NULL);
+	
+	  /* -------------------------------------------------------------------------- */
+	
+	  self.catlabel2 = gtk_label_new (NULL); 
+	  fstr = g_strdup_printf("%s %s %s","<b>",_("Time"),"</b>");
+	  gtk_label_set_markup (GTK_LABEL(self.catlabel2),fstr); 
+	  g_free(fstr);
+	  gtk_table_attach (GTK_TABLE (table), self.catlabel2, 0, 3, 2, 3,
+				GTK_FILL,GTK_FILL  | GTK_EXPAND,0,0);
+	  gtk_misc_set_alignment (GTK_MISC (self.catlabel2), 0.0, 0.9);
+	
+	  self.tsel = gpe_time_sel_new();
+	  gpe_time_sel_set_time(GPE_TIME_SEL(self.tsel),(guint)ts.tm_hour, (guint)ts.tm_min);
+	  gtk_table_attach (GTK_TABLE (table), self.tsel, 0, 3, 3, 4,
+				GTK_FILL,0,3,0);
+	  /* -------------------------------------------------------------------------- */
+	
+	  self.catlabel3 = gtk_label_new (NULL);
+	  fstr = g_strdup_printf("%s %s %s","<b>",_("Network"),"</b>");
+	  gtk_label_set_markup (GTK_LABEL(self.catlabel3),fstr); 
+	  g_free(fstr);
+	  
+	  gtk_table_attach (GTK_TABLE (table), self.catlabel3, 0, 3, 4, 5,
+				GTK_FILL,GTK_FILL  | GTK_EXPAND,0,0);
+	  gtk_misc_set_alignment (GTK_MISC (self.catlabel3), 0.0, 0.9);
+	
+	  self.ntpserver = gtk_combo_new ();
+	
+	  for (idx=0; idx<5; idx++) {
+		ntpsrv = g_list_append (ntpsrv, Ntpservers[idx]);
+	  }
+	
+	  gtk_combo_set_popdown_strings (GTK_COMBO (self.ntpserver), ntpsrv);
+	  gtk_table_attach (GTK_TABLE (table), self.ntpserver, 0, 3, 5, 6,
+				GTK_FILL,0,3,0);
+	  
+	  gtk_tooltips_set_tip (tooltips, self.ntpserver, _("Select the timeserver to use to set the clock."), NULL);
+	
+	  self.internet = gtk_button_new_with_label(_("Get time from network"));
+	  gtk_signal_connect (GTK_OBJECT(self.internet), "clicked",
+				  (GtkSignalFunc) GetInternetTime, NULL);
+	  gtk_widget_set_sensitive(self.internet,is_network_up());
+	  
+	  gtk_table_attach (GTK_TABLE (table), self.internet, 0, 3, 6, 7,
+				GTK_FILL,0,gpe_border,gpe_border);
+	  gtk_tooltips_set_tip (tooltips, self.internet, _("If connected to the Internet, you may press this button to set the time on this device using the timeserver above."), NULL);
+  } // root_mode
   
    /*------------------------------*/
 
   label = gtk_label_new(_("Timezone"));
-  self.catvbox4 = gtk_vbox_new (FALSE, gpe_boxspacing);
+  self.catvbox4 = table = gtk_table_new (8, 3, FALSE);
+  gtk_container_set_border_width (GTK_CONTAINER (table), gpe_border);
+  gtk_table_set_row_spacings (GTK_TABLE (table), gpe_boxspacing);
+  gtk_table_set_col_spacings (GTK_TABLE (table), gpe_boxspacing);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),self.catvbox4,label);
 
   self.catlabel4 = gtk_label_new (NULL);
-  fstr = g_strdup_printf("%s %s %s","<b>",_("Timezone"),"</b>");
+  fstr = g_strdup_printf("<b>%s</b>",_("Timezone"));
   gtk_label_set_markup (GTK_LABEL(self.catlabel4),fstr); 
   g_free(fstr);
   
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), self.catlabel4, FALSE, FALSE, 0);
   gtk_misc_set_alignment (GTK_MISC (self.catlabel4), 0.0, 0.9);
-
-  self.catconthbox4 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), self.catconthbox4, FALSE, TRUE, 0);
-
-  self.catindentlabel4 = gtk_label_new (gpe_catindent);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox4), self.catindentlabel4, FALSE, TRUE, 0);
-
-  self.controlvbox4 = gtk_vbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.catconthbox4), self.controlvbox4, TRUE, TRUE, 0);
-
+  gtk_table_attach (GTK_TABLE (table), self.catlabel4, 0, 3, 0, 1,
+		    GTK_FILL,GTK_FILL | GTK_EXPAND,0,0);
 
   self.timezone = gtk_combo_new ();
   tzi = get_tz_info(getenv("TZ"));
@@ -496,11 +539,6 @@ GtkWidget *Time_Build_Objects()
   else
     fstr=g_strdup("UTC");
   
-  printf("TZ.name = %s\n",tzi.tzname);
-  printf("TZ.dstname = %s\n",tzi.dstname);
-  printf("TZ.utcofs_h = %i\n",tzi.utcofs_h);
-  printf("TZ.utcdstofs_h = %i\n",tzi.utcdstofs_h);
-
   for (idx=0; idx<TZ_MAXINDEX; idx++) {
     tzones = g_list_append (tzones, timezones[idx][0]);
     if (strstr(timezones[idx][0],fstr)) mark=idx;
@@ -508,8 +546,9 @@ GtkWidget *Time_Build_Objects()
 
   g_free(fstr);
   gtk_combo_set_popdown_strings (GTK_COMBO (self.timezone), tzones);
-  gtk_box_pack_start(GTK_BOX(self.controlvbox4), self.timezone, FALSE, FALSE, 0); 
-  gtk_tooltips_set_tip (tooltips, self.timezone, _("Select your current timezone here. The setting applies after after next login."), NULL);
+  gtk_tooltips_set_tip (tooltips, self.timezone, _("Select your current timezone here. The setting applies after the next login."), NULL);
+  gtk_table_attach (GTK_TABLE (table), self.timezone, 0, 3, 1, 2,
+		    GTK_FILL,0,gpe_boxspacing,0);
 
   gtk_list_select_item(GTK_LIST(GTK_COMBO(self.timezone)->list),mark);
 
@@ -518,25 +557,32 @@ GtkWidget *Time_Build_Objects()
   fstr = g_strdup_printf("<b>%s</b>",_("Daylight Saving"));
   gtk_label_set_markup (GTK_LABEL(label),fstr); 
   g_free(fstr);
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), label, FALSE, TRUE, 0);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 3, 2, 3,
+		    GTK_FILL,GTK_FILL  | GTK_EXPAND,0,0);
 
   /* ---- */  
   self.usedst = gtk_check_button_new_with_label(_("Use daylight saving time"));
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), self.usedst, FALSE, TRUE, 0);
+  gtk_table_attach (GTK_TABLE (table), self.usedst, 0, 3, 3, 4,
+		    GTK_FILL,0,gpe_boxspacing,0);
+  
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self.usedst), strlen(tzi.dstname));
   g_signal_connect (G_OBJECT(self.usedst), "toggled",
 		      G_CALLBACK(update_enabled_widgets), NULL);
   gtk_tooltips_set_tip (tooltips, self.usedst, _("Check this box if you want your device to handle daylight saving time for you."), NULL);
 
-  self.offsetlabel = gtk_label_new(_("Offset"));
+  self.offsetlabel = gtk_label_new(_("Offset:"));
   gtk_misc_set_alignment(GTK_MISC(self.offsetlabel),0.0,0.5);
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), self.offsetlabel, FALSE, TRUE, 0);
+  gtk_table_attach (GTK_TABLE (table), self.offsetlabel, 0, 1, 4, 5,
+		    GTK_FILL,GTK_FILL,gpe_boxspacing,0);
 
   hbox = gtk_hbox_new (FALSE, gpe_boxspacing);
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), hbox, FALSE, TRUE, 0);
-    
+  gtk_table_attach (GTK_TABLE (table), hbox, 0, 3, 5, 6,
+		    GTK_FILL,0,gpe_boxspacing,0);
+      
   self.defaultdst = gtk_check_button_new_with_label(_("Default"));
-  gtk_box_pack_start (GTK_BOX (hbox), self.defaultdst, FALSE, TRUE, 0);
+  gtk_table_attach (GTK_TABLE (table), self.defaultdst, 1, 3, 4, 5,
+		    GTK_FILL,0,gpe_boxspacing,0);
+			
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self.defaultdst), !(tzi.utcdstofs_h || tzi.utcdstofs_m));
   g_signal_connect (G_OBJECT(self.defaultdst), "toggled",
 		      G_CALLBACK(update_enabled_widgets), NULL);
@@ -544,10 +590,10 @@ GtkWidget *Time_Build_Objects()
 
   adj = gtk_adjustment_new(tzi.utcdstofs_h,-12,12,1,6,6);
   self.dsth = gtk_spin_button_new (GTK_ADJUSTMENT(adj),1,0);
-  gtk_box_pack_start (GTK_BOX (hbox), self.dsth, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), self.dsth, FALSE, TRUE, gpe_boxspacing);
   self.dstlh = gtk_label_new(_("Hours"));
   gtk_misc_set_alignment(GTK_MISC(self.dstlh),0.0,0.5);
-  gtk_box_pack_start (GTK_BOX (hbox), self.dstlh, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), self.dstlh, FALSE, TRUE, gpe_boxspacing);
   gtk_tooltips_set_tip (tooltips, self.dsth, _("Select daylight saving time offset hours here."), NULL);
 
   adj = gtk_adjustment_new(tzi.utcdstofs_m,0,55,5,15,15);
@@ -559,23 +605,27 @@ GtkWidget *Time_Build_Objects()
   gtk_tooltips_set_tip (tooltips, self.dstm, _("Select daylight saving time offset minutes here."), NULL);
   
   /* ---- */
-  
+/*  
   label = gtk_label_new (NULL);
   gtk_misc_set_alignment(GTK_MISC(label),0.0,0.9);
   fstr = g_strdup_printf("<b>%s</b>",_("Advanced Settings"));
   gtk_label_set_markup (GTK_LABEL(label),fstr); 
   g_free(fstr);
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), label, FALSE, TRUE, 0);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 3, 7, 8,
+		    GTK_FILL,GTK_FILL  | GTK_EXPAND,0,0);
   
   self.advanced = gtk_check_button_new_with_label(_("Use advanced settings"));
-  gtk_box_pack_start (GTK_BOX (self.catvbox4), self.advanced, FALSE, TRUE, 0);
+  gtk_table_attach (GTK_TABLE (table), self.advanced, 0, 3, 8, 9,
+		    GTK_FILL,0,gpe_boxspacing,0);
   gtk_tooltips_set_tip (tooltips, self.advanced, _("Check this if you want to use the advanced settings below."), NULL);
   
-  
+*/  
   /* -------------------------------------------------------------------------- */
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self.usedst),strlen(tzi.dstname));
 
   update_enabled_widgets(NULL);
+  g_signal_connect_after (G_OBJECT (notebook), "switch-page",
+			G_CALLBACK (do_tabchange), NULL);
   
   return notebook;
 }
@@ -591,27 +641,15 @@ void Time_Save()
   time_t t;
   char* par = malloc(100);
   tzinfo tz;
-  
-  gtk_calendar_get_date(GTK_CALENDAR(GTK_DATE_COMBO(self.cal)->cal),&year,&month,&day);
-  h=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(self.h));
-  m=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(self.m));
-  s=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(self.s));
-  tm.tm_mday=day;
-  tm.tm_mon=month;
-  tm.tm_year=year-1900;
-  tm.tm_hour=h;
-  tm.tm_min=m;
-  tm.tm_sec=s;
-  tm.tm_isdst = 0;
-  t=mktime(&tm);
+  GtkWidget *dialog;	
+  char* fprof;
 
-  /* set time */
-  snprintf(par,99,"%ld",t);
-  suid_exec("STIM",par);
+  if (geteuid()) 
+	  fprof = g_strdup_printf("%s/.profile",g_get_home_dir());
+  else
+	  fprof = g_strdup("/etc/profile");	  
   
-  free(par);
-  
-  /* set timezone */
+   /* set timezone */
     
   for (s=0;s<=TZ_MAXINDEX;s++)
 	  if (!strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(self.timezone)->entry)),timezones[s][0]))
@@ -632,28 +670,74 @@ void Time_Save()
 			tz.utcdstofs_m = 0;
 		}
 		par = compose_tz_advanced(tz);
-		printf("TZ = %s\n",par);
-  		if (setenv("TZ",par,1))
-	  		gpe_error_box(_("Could not set timezone."));
-		suid_exec("STZO",par);
+		
+		set_timezone(fprof,par);
 		break;
 	  }
+  
+  if (!nonroot_mode)
+  {
+	  tz = get_tz_info(par);
+		  
+	  gtk_calendar_get_date(GTK_CALENDAR(GTK_DATE_COMBO(self.cal)->cal),&year,&month,&day);
+	  gpe_time_sel_get_time(GPE_TIME_SEL(self.tsel),&h,&m);
+	  s = 0;	 
+	  tm.tm_mday=day;
+	  tm.tm_mon=month;
+	  tm.tm_year=year-1900;
+	  if (isdst)
+	  {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self.defaultdst)))
+		{
+		  tm.tm_hour=(h+24+(tz.utcofs_h+24-1)%24) % 24; // defaults to one hour ahead
+		  tm.tm_min=(m+60+tz.utcofs_m) % 60;
+		}
+		else
+		{
+		  tm.tm_hour=(h+24+tz.utcdstofs_h) % 24;
+		  tm.tm_min=(m+60+tz.utcdstofs_m) % 60;
+		}
+	
+	  }
+	  else
+	  {
+		 tm.tm_hour=(h+24+tz.utcofs_h) % 24;
+		 tm.tm_min=(m+60+tz.utcofs_m) % 60;
+	  }
+	  tm.tm_sec=s;
+	  tm.tm_isdst = isdst;
+	  t = timegm(&tm);
+	  
+	  /* set time */ 
+	  snprintf(par,99,"%ld",t);
+	  suid_exec("STIM",par);  
+	  free(par);  
+  }
+  if (need_warning)
+  {
+	dialog = gtk_message_dialog_new (GTK_WINDOW (mainw),
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_WARNING,
+						 GTK_BUTTONS_OK,
+						 _("To make timezone settings take effect, you'll need to log out and log in again."));
+    gtk_dialog_run(GTK_DIALOG(dialog));
+  }
 }
 
+
 void Time_Restore()
 {
   time_t t;
   struct tm *tsptr;
   struct tm ts;     /* gtk_cal seems to modify the ptr
 				returned by localtime, so we duplicate it.. */
-  
-  time(&t);
-  tsptr = localtime(&t);
-  ts = *tsptr;
-  gtk_calendar_select_month(GTK_CALENDAR(GTK_DATE_COMBO(self.cal)->cal),ts.tm_mon,ts.tm_year+1900);
-  gtk_calendar_select_day(GTK_CALENDAR(GTK_DATE_COMBO(self.cal)->cal),ts.tm_mday);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(self.h),ts.tm_hour);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(self.m),ts.tm_min);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(self.s),ts.tm_sec);
-
+  if (!nonroot_mode)
+  {
+    time(&t);
+    tsptr = localtime(&t);
+    ts = *tsptr;
+    gtk_calendar_select_month(GTK_CALENDAR(GTK_DATE_COMBO(self.cal)->cal),ts.tm_mon,ts.tm_year+1900);
+    gtk_calendar_select_day(GTK_CALENDAR(GTK_DATE_COMBO(self.cal)->cal),ts.tm_mday);
+    gpe_time_sel_set_time(GPE_TIME_SEL(self.tsel),(guint)ts.tm_hour, (guint)ts.tm_min);
+  }
 }
